@@ -8,7 +8,7 @@ This module exists purely for other languages to interface with mwalib.
 
 use crate::*;
 use gpubox_files::GpuboxError;
-use libc::{c_char, c_float, c_uchar, c_uint, c_ulong, size_t};
+use libc::{c_char, c_double, c_float, c_uchar, c_uint, c_ulong, size_t};
 use std::ffi::*;
 use std::mem;
 use std::slice;
@@ -602,6 +602,206 @@ pub unsafe extern "C" fn mwalib_correlator_context_read_by_frequency(
     }
 }
 
+/// For a given slice of correlator coarse channel indices, return a vector of the center
+/// frequencies for all the fine channels in the given coarse channels
+///
+/// # Arguments
+///
+/// * `correlator_context_ptr` - pointer to an already populated `CorrelatorContext` object.
+///
+/// * `corr_coarse_chan_indices_array_ptr` - a pointer to an array containing correlator coarse channel indices
+///                                          for which you want fine channels for. Does not need to be
+///                                          contiguous.
+///
+/// * `corr_coarse_chan_indices_array_len` - length of `corr_coarse_chan_indices_array_ptr`.
+///
+/// * `out_fine_chan_freq_array_ptr` - pointer to caller-owned and allocated array of doubles to write frequencies into.
+///
+/// * `out_fine_chan_freq_array_len` - length of `out_fine_chan_freq_array_ptr`.
+///
+/// * `error_message` - pointer to already allocated buffer for any error messages to be returned to the caller.
+///
+/// * `error_message_length` - length of error_message char* buffer.
+///
+///
+/// # Safety
+/// * `error_message` *must* point to an already allocated char* buffer for any error messages.
+/// * `correlator_context_ptr` must point to a populated object from the `mwalib_correlator_context_new` function.
+/// * Caller *must* call `mwalib_correlator_context_free_read_buffer` function to release the rust memory.
+#[no_mangle]
+pub unsafe extern "C" fn mwalib_correlator_context_get_fine_chan_freqs_hz_array(
+    correlator_context_ptr: *mut CorrelatorContext,
+    corr_coarse_chan_indices_array_ptr: *mut size_t,
+    corr_coarse_chan_indices_array_len: size_t,
+    out_fine_chan_freq_array_ptr: *mut c_double,
+    out_fine_chan_freq_array_len: size_t,
+    error_message: *const c_char,
+    error_message_length: size_t,
+) -> i32 {
+    // Load the previously-initialised context and buffer structs. Exit if
+    // either of these are null.
+    let corr_context = if correlator_context_ptr.is_null() {
+        set_error_message(
+            "mwalib_correlator_context_get_fine_chan_freqs_hz_array() ERROR: null pointer for correlator_context_ptr passed in",
+            error_message as *mut u8,
+            error_message_length,
+        );
+        return MWALIB_FAILURE;
+    } else {
+        &mut *correlator_context_ptr
+    };
+
+    // Don't do anything if the input pointer is null.
+    if corr_coarse_chan_indices_array_ptr.is_null() {
+        set_error_message(
+            "mwalib_correlator_context_get_fine_chan_freqs_hz_array() ERROR: null pointer for corr_coarse_chan_indices_array_ptr passed in",
+            error_message as *mut u8,
+            error_message_length,
+        );
+        return MWALIB_FAILURE;
+    }
+
+    // Get input buffer ready to be passed into rust method
+    let input_coarse_chan_indices = slice::from_raw_parts_mut(
+        corr_coarse_chan_indices_array_ptr,
+        corr_coarse_chan_indices_array_len,
+    );
+
+    // Don't do anything if the buffer pointer is null.
+    if out_fine_chan_freq_array_ptr.is_null() {
+        set_error_message(
+            "mwalib_correlator_context_get_fine_chan_freqs_hz_array() ERROR: null pointer for out_fine_chan_freq_array_ptr passed in",
+            error_message as *mut u8,
+            error_message_length,
+        );
+        return MWALIB_FAILURE;
+    }
+
+    // Get output buffer ready
+    let output_slice =
+        slice::from_raw_parts_mut(out_fine_chan_freq_array_ptr, out_fine_chan_freq_array_len);
+
+    // Sanity check the length
+    let expected_output_len = corr_coarse_chan_indices_array_len
+        * corr_context.metafits_context.num_corr_fine_chans_per_coarse;
+    if output_slice.len() != expected_output_len {
+        set_error_message(
+            &format!("mwalib_correlator_context_get_fine_chan_freqs_hz_array() ERROR: number of elements in out_fine_chan_freq_array_ptr does not match expected value {}", expected_output_len),
+            error_message as *mut u8,
+            error_message_length,
+        );
+        return MWALIB_FAILURE;
+    }
+
+    // Read data into provided buffer
+    let fine_chans = corr_context.get_fine_chan_freqs_hz_array(input_coarse_chan_indices);
+
+    // Write the fine chans back into the provided array
+    output_slice.clone_from_slice(&fine_chans);
+
+    MWALIB_SUCCESS
+}
+
+/// For a given slice of voltage coarse channel indices, return a vector of the center
+/// frequencies for all the fine channels in the given coarse channels
+///
+/// # Arguments
+///
+/// * `voltage_context_ptr` - pointer to an already populated `VoltageContext` object.
+///
+/// * `corr_coarse_chan_indices_array_ptr` - a pointer to an array containing correlator coarse channel indices
+///                                          for which you want fine channels for. Does not need to be
+///                                          contiguous.
+///
+/// * `corr_coarse_chan_indices_array_len` - length of `corr_coarse_chan_indices_array_ptr`.
+///
+/// * `out_fine_chan_freq_array_ptr` - pointer to caller-owned and allocated array of doubles to write frequencies into.
+///
+/// * `out_fine_chan_freq_array_len` - length of `out_fine_chan_freq_array_ptr`.
+///
+/// * `error_message` - pointer to already allocated buffer for any error messages to be returned to the caller.
+///
+/// * `error_message_length` - length of error_message char* buffer.
+///
+///
+/// # Safety
+/// * `error_message` *must* point to an already allocated char* buffer for any error messages.
+/// * `correlator_context_ptr` must point to a populated object from the `mwalib_correlator_context_new` function.
+/// * Caller *must* call `mwalib_correlator_context_free_read_buffer` function to release the rust memory.
+#[no_mangle]
+pub unsafe extern "C" fn mwalib_voltage_context_get_fine_chan_freqs_hz_array(
+    voltage_context_ptr: *mut VoltageContext,
+    volt_coarse_chan_indices_array_ptr: *mut size_t,
+    volt_coarse_chan_indices_array_len: size_t,
+    out_fine_chan_freq_array_ptr: *mut c_double,
+    out_fine_chan_freq_array_len: size_t,
+    error_message: *const c_char,
+    error_message_length: size_t,
+) -> i32 {
+    // Load the previously-initialised context and buffer structs. Exit if
+    // either of these are null.
+    let volt_context = if voltage_context_ptr.is_null() {
+        set_error_message(
+            "mwalib_voltage_context_get_fine_chan_freqs_hz_array() ERROR: null pointer for voltage_context_ptr passed in",
+            error_message as *mut u8,
+            error_message_length,
+        );
+        return MWALIB_FAILURE;
+    } else {
+        &mut *voltage_context_ptr
+    };
+
+    // Don't do anything if the input pointer is null.
+    if volt_coarse_chan_indices_array_ptr.is_null() {
+        set_error_message(
+            "mwalib_voltage_context_get_fine_chan_freqs_hz_array() ERROR: null pointer for volt_coarse_chan_indices_array_ptr passed in",
+            error_message as *mut u8,
+            error_message_length,
+        );
+        return MWALIB_FAILURE;
+    }
+
+    // Get input buffer ready to be passed into rust method
+    let input_coarse_chan_indices = slice::from_raw_parts_mut(
+        volt_coarse_chan_indices_array_ptr,
+        volt_coarse_chan_indices_array_len,
+    );
+
+    // Don't do anything if the buffer pointer is null.
+    if out_fine_chan_freq_array_ptr.is_null() {
+        set_error_message(
+            "mwalib_voltage_context_get_fine_chan_freqs_hz_array() ERROR: null pointer for out_fine_chan_freq_array_ptr passed in",
+            error_message as *mut u8,
+            error_message_length,
+        );
+        return MWALIB_FAILURE;
+    }
+
+    // Get output buffer ready
+    let output_slice =
+        slice::from_raw_parts_mut(out_fine_chan_freq_array_ptr, out_fine_chan_freq_array_len);
+
+    // Sanity check the length
+    let expected_output_len = volt_coarse_chan_indices_array_len
+        * volt_context.metafits_context.num_corr_fine_chans_per_coarse;
+    if output_slice.len() != expected_output_len {
+        set_error_message(
+            &format!("mwalib_voltage_context_get_fine_chan_freqs_hz_array() ERROR: number of elements in out_fine_chan_freq_array_ptr does not match expected value {}", expected_output_len),
+            error_message as *mut u8,
+            error_message_length,
+        );
+        return MWALIB_FAILURE;
+    }
+
+    // Read data into provided buffer
+    let fine_chans = volt_context.get_fine_chan_freqs_hz_array(input_coarse_chan_indices);
+
+    // Write the fine chans back into the provided array
+    output_slice.clone_from_slice(&fine_chans);
+
+    MWALIB_SUCCESS
+}
+
 /// Free a previously-allocated `CorrelatorContext` struct (and it's members).
 ///
 /// # Arguments
@@ -995,9 +1195,9 @@ pub struct MetafitsMetadata {
     pub moon_distance_deg: f64,
     /// Distance from pointing center to Jupiter
     pub jupiter_distance_deg: f64,
-    /// Local Sidereal Time
+    /// Local Sidereal Time in degrees (at the midpoint of the observation)
     pub lst_deg: f64,
-    /// Local Sidereal Time in radians
+    /// Local Sidereal Time in radians (at the midpoint of the observation)        
     pub lst_rad: f64,
     /// Hour Angle of pointing center (as a string)
     pub hour_angle_string: *mut c_char,
@@ -1025,6 +1225,10 @@ pub struct MetafitsMetadata {
     pub corr_int_time_ms: u64,
     /// Number of fine channels in each coarse channel for a correlator observation
     pub num_corr_fine_chans_per_coarse: usize,
+    /// Voltage fine_chan_resolution
+    pub volt_fine_chan_width_hz: u32,
+    /// Number of fine channels in each coarse channel for a voltage observation
+    pub num_volt_fine_chans_per_coarse: usize,
     /// RECVRS    Array of receiver numbers (this tells us how many receivers too)
     pub receivers: *mut usize,
     /// DELAYS    Array of delays
@@ -1073,6 +1277,10 @@ pub struct MetafitsMetadata {
     pub num_metafits_coarse_chans: usize,
     /// metafits_coarse_chans array
     pub metafits_coarse_chans: *mut CoarseChannel,
+    /// Number of fine channels for the whole observation
+    pub num_metafits_fine_chan_freqs: usize,
+    /// Vector of fine channel frequencies for the whole observation
+    pub metafits_fine_chan_freqs: *mut f64,
     /// Number of timesteps based on the metafits
     pub num_metafits_timesteps: usize,
     /// metafits_timesteps array
@@ -1330,6 +1538,8 @@ pub unsafe extern "C" fn mwalib_metafits_metadata_get(
             corr_fine_chan_width_hz,
             corr_int_time_ms,
             num_corr_fine_chans_per_coarse,
+            volt_fine_chan_width_hz,
+            num_volt_fine_chans_per_coarse,
             receivers,
             delays,
             global_analogue_attenuation_db,
@@ -1346,6 +1556,8 @@ pub unsafe extern "C" fn mwalib_metafits_metadata_get(
             num_visibility_pols,
             metafits_timesteps: _, // This is populated seperately
             num_metafits_timesteps,
+            metafits_fine_chan_freqs,
+            num_metafits_fine_chan_freqs,
             metafits_coarse_chans: _, // This is populated seperately
             num_metafits_coarse_chans,
             obs_bandwidth_hz,
@@ -1387,6 +1599,8 @@ pub unsafe extern "C" fn mwalib_metafits_metadata_get(
             corr_fine_chan_width_hz: *corr_fine_chan_width_hz,
             corr_int_time_ms: *corr_int_time_ms,
             num_corr_fine_chans_per_coarse: *num_corr_fine_chans_per_coarse,
+            volt_fine_chan_width_hz: *volt_fine_chan_width_hz,
+            num_volt_fine_chans_per_coarse: *num_volt_fine_chans_per_coarse,
             receivers: ffi_array_to_boxed_slice(receivers.clone()),
             delays: ffi_array_to_boxed_slice(delays.clone()),
             sched_start_utc: sched_start_utc.timestamp(),
@@ -1411,6 +1625,8 @@ pub unsafe extern "C" fn mwalib_metafits_metadata_get(
             num_visibility_pols: *num_visibility_pols,
             num_metafits_coarse_chans: *num_metafits_coarse_chans,
             metafits_coarse_chans: ffi_array_to_boxed_slice(coarse_chan_vec),
+            num_metafits_fine_chan_freqs: *num_metafits_fine_chan_freqs,
+            metafits_fine_chan_freqs: ffi_array_to_boxed_slice(metafits_fine_chan_freqs.clone()),
             num_metafits_timesteps: *num_metafits_timesteps,
             metafits_timesteps: ffi_array_to_boxed_slice(timestep_vec),
             obs_bandwidth_hz: *obs_bandwidth_hz,
@@ -1517,6 +1733,13 @@ pub unsafe extern "C" fn mwalib_metafits_metadata_free(
         drop(Box::from_raw((*metafits_metadata_ptr).delays));
     }
 
+    // fine channel freqs
+    if !(*metafits_metadata_ptr).metafits_fine_chan_freqs.is_null() {
+        drop(Box::from_raw(
+            (*metafits_metadata_ptr).metafits_fine_chan_freqs,
+        ));
+    }
+
     // Free main metadata struct
     drop(Box::from_raw(metafits_metadata_ptr));
 
